
import { Subject } from 'rxjs';
import { tap, map } from 'rxjs/operators';
import Setting from '@neux/ui-jquery/config.json';
import { formViewGenerator, createForm, formButtonViewGenerator } from './form';
import { getDefaultValueFromConfig, replaceAll } from './utils';
import { addScript, addStyle } from './add-script';

const SCSS_SERVER = 'https://labs.decoder.com.tw/scss-compiler/compiler/scss';
const IMG_PATH = 'static/img';

const CONFIG_FORM_BUTTON = [{
  type: 'submit',
  key: 'apply',
  name: 'Apply'
}, {
  type: 'button',
  key: 'export',
  name: 'Export Argument'
}];


class DynamicLoader {
  constructor(configTarget, mainTarget, listTarget, subject = new Subject()) {
    if (!configTarget) {
      throw new Error('Config target not valid!');
    }
    if (!mainTarget) {
      throw new Error('main Target not valid! ');
    }

    this.configTarget = configTarget;
    this.mainTarget = mainTarget;
    this.listTarget = listTarget;
    this.current$ = subject;
    console.log(Setting);
    this.setting = Setting;
    this.init();
  }

  /**
   * 初始化Dynamic loader
   *
   * @memberof DynamicLoader
   */
  init() {
    // subscribe on change
    this.current$.pipe(
      map(x => this.getConfig(x)),
      tap(x => console.log(x)),
      tap(x => this.currentConfig = x),
      tap(x => this.renderConfigArea(x))
    ).subscribe();

    // generate list
    Object.entries(this.setting)
      .map(([key]) => $(`<li name="${key}" type="component">${key}</li>`))
      .reduce((p, n) => p.append(n), this.listTarget);
    this.bindEvent();

  }

  /**
   * 根據傳入的type/name，取得config
   *
   * @param {type:string,name:string} selectedType
   * @returns {type:string,name:string, data:any} 
   * @memberof DynamicLoader
   */
  getConfig(selectedType) {
    const { type, name } = selectedType;
    if (!type || !name) {
      throw new Error(`getConfig Error, Incorrect type ${type} or name ${name}.`);
    }

    return {
      ...selectedType,
      data: this.setting[name]
    };
  }

  /**
   * 根據Config，長出左側form html
   *
   * @memberof DynamicLoader
   */
  renderConfigArea(config) {
    const jsOptions = config.data?.js?.config || [];
    const cssOptions = config.data?.css?.config || [];
    const formConfig = [...jsOptions, ...cssOptions];
    const buttonConfig = CONFIG_FORM_BUTTON;
    let form = formViewGenerator(formConfig, { formClass: 'config-form' });
    let button = formButtonViewGenerator(buttonConfig);
    form.append(button);
    this.configTarget.empty();
    this.configTarget.append(form);
    this.formObject = createForm(form, (value) => this.onApply(value), getDefaultValueFromConfig(formConfig), null);


  }


  /**
   * 綁定按鈕事件
   *
   * @memberof DynamicLoader
   */
  bindEvent() {
    $(document).on('click', 'form button.action__export', (e) => this.onExportParams(e));
    $(document).on('click', '.component-list>li', (e) => {
      const _this = $(e.target);
      const name = _this.attr('name');
      const type = _this.attr('type');
      this.current$.next({ type, name });
    })
  }

  /**
   * Apply現在設定並preview
   *
   * @param {*} value
   * @memberof DynamicLoader
   */
  onApply(value) {
    console.log("DynamicLoader -> onApply -> value", value)

    if (!this.currentConfig) {
      throw new Error('CurrentConfig not set');
    }

    let cssCaller = this.currentConfig?.data?.css?.caller;
    let jsCaller = this.currentConfig?.data?.js?.caller;
    const template = this.currentConfig?.data?.html;

    // replace variables
    Object.entries({ ...value, target: '.demo-target' }).forEach((val) => {
      const [key, value] = val;
      if (cssCaller)
        cssCaller = replaceAll(cssCaller, `\${${key}}`, value);
      if (jsCaller)
        jsCaller = replaceAll(jsCaller, `\${${key}}`, value);
    });

    if (this.currentConfig.type === 'component') {
      this.onComponentApply({ cssCaller, jsCaller, template, config: this.currentConfig });
    }

  }

  /**
   * 當apply的元件為component時呼叫
   *
   * @param {*} options
   * @memberof DynamicLoader
   */
  async onComponentApply(options) {
    let { cssCaller, jsCaller, template, config } = options;

    // Add html
    let templateObj = $(template).addClass('demo-target');
    this.mainTarget.empty();
    this.mainTarget.append(templateObj);

    // remove old js/css
    $('#dynamicScript').remove();
    $('#dynamicStyle').remove();


    // compile css
    cssCaller = `@import "main"; @import "${config.name}";@include reset(); ${cssCaller}`;
    let cssStyle = await this.compileScss(cssCaller);

    // replace scss img path
    cssStyle = replaceAll(cssStyle, '~@img', IMG_PATH);

    // add script/style
    setTimeout(() => {
      addStyle($('head'), cssStyle);
      addScript($('body'), jsCaller);
    }, 100);
  }

  /**
   * 呼叫scss server編譯scss成css
   *
   * @param {*} source
   * @returns
   * @memberof DynamicLoader
   */
  async compileScss(source) {
    return new Promise((res, rej) => {
      $.ajax({
        type: "POST",
        url: SCSS_SERVER,
        data: JSON.stringify({ scss: source }),
        contentType: 'application/json;charset=UTF-8'
      })
        .done((data) => {
          res(data)
        })
        .fail(err => {
          rej(err)
        });
    })

  }

  /**
   * 當Apply的元件為Exhibition時呼叫
   *
   * @memberof DynamicLoader
   */
  onExhibitionApply() { }

  /**
   * 匯出參數成JSON
   *
   * @memberof DynamicLoader
   */
  onExportParams() {
    if (!this.formObject) {
      throw new Error("Form object not setted.");
    }
    // get data, create element and download

    let unsubscribe = this.formObject.subscribe((state) => {
      console.log("onExportParams -> state", state)
      let dataString = `data:text/json;charset=utf-8, ${encodeURIComponent(JSON.stringify(state.values))}`;
      let downloadLink = $(`<a href="${dataString}" download="params.json"></a>`).appendTo('body');
      downloadLink[0].click();
      downloadLink.remove();
    }, {
      values: true,
    });
    unsubscribe();
  }

  export() { }
}


export {
  DynamicLoader
}