import { createForm as FcreateForm } from 'final-form'

/**
 * Create final-form object by passing form elements
 *
 * @param {*} form
 * @param {*} onSubmit
 * @param {*} initialValues
 * @param {*} validator
 * @returns
 */
const createForm = (form, onSubmit, initialValues, validator) => {
  console.log("createForm -> initialValues", initialValues)


  const formObject = FcreateForm({
    onSubmit,
    initialValues,
    validate: validator
  });

  $(form).on('submit', e => {
    e.preventDefault();
    formObject.submit();
  });

  $(form).find('input,select').each((index, element) => {
    if (element.name)
      registerField(formObject, element);
  });

  return formObject;
}

const registerField = (form, input) => {
  const { name } = input;
  form.registerField(
    name,
    fieldState => {
      const { blur, change, error, focus, touched, value } = fieldState;
      const errorElement = document.getElementById(name + '_error')

      // first time, register event listeners
      input.addEventListener('blur', () => blur())
      input.addEventListener('input', event =>
        change(
          input.type === 'checkbox'
            ? event.target.checked
            : event.target.value
        )
      )
      input.addEventListener('focus', () => focus())


      // update value
      if (input.type === 'checkbox') {
        input.checked = value
      } else {
        input.value = value === undefined ? '' : value
      }

      // show/hide errors
      if (errorElement) {
        if (touched && error) {
          errorElement.innerHTML = error
          errorElement.style.display = 'block'
        } else {
          errorElement.innerHTML = ''
          errorElement.style.display = 'none'
        }
      }
    },
    {
      value: true,
      error: true,
      touched: true
    }
  )
}

/**
 * Form View Generator
 *
 * @param {*} config
 * @param {*} [opt={}]
 * @returns
 */
const formViewGenerator = (configList, opt = {}) => {
  const options = $.extend({
    formClass: 'config-form',
  }, opt);
  const form = $(`
  <form class="${options.formClass}">
  <ul class="config-list list list--bordered">
  </ul>
  </form>
  `);
  configList.map(col => fieldViewGenerator(col)).reduce(
    (prev, next) => {
      prev.children('.config-list').append(next);
      return prev;
    }, form
  );
  return form;
}

/**
 * Form Button View Generator
 *
 * @param {*} config
 */
const formButtonViewGenerator = (configList) => {
  const buttonSection = $(`<div class="form__action"></div>`);
  configList.map(config => buttonViewGenerator(config)).reduce(
    (prev, next) => {
      prev.append(next);
      return prev;
    }, buttonSection
  );
  return buttonSection;
}

/**
 * Form Filed View Generator
 *
 * @param {*} columnConfig
 * @returns
 */
const fieldViewGenerator = (columnConfig) => {
  const type = columnConfig.type?.toLowerCase();
  const list = $(`
  <li class="list__item item">
    <div class="item__title">${columnConfig.name}</div>
    <div class="item_description">${columnConfig.description}</div>
    <div class="item__content"></div>
  </li>
  `);
  let innerElement;
  if (type === 'input') {
    innerElement = inputViewGenerator(columnConfig);
  }
  else if (type === 'range') {
    innerElement = rangeViewGenerator(columnConfig);
  }
  else if (type === 'slidetoggle') {
    innerElement = switchViewGenerator(columnConfig);
  }
  else if (type === 'select') {
    innerElement = selectViewGenerator(columnConfig);
  }

  list.children('.item__content').append(innerElement);
  return list;

}

/**
 * Form Input View Generator
 *
 * @param {*} columnConfig
 * @returns
 */
const inputViewGenerator = (columnConfig) => {
  const name = columnConfig.key;
  return $(`<input type="text" name="${name}">`);
}

/**
 * Form Range View Generator
 *
 * @param {*} columnConfig
 * @returns
 */
const rangeViewGenerator = (columnConfig) => {
  const name = columnConfig.key;
  return $(`<input type="range" min="1" max="100" value="50" class="slider" name="${name}">`);
}

/**
 * Form Switch View Generator
 *
 * @param {*} columnConfig
 * @returns
 */
const switchViewGenerator = (columnConfig) => {
  const name = columnConfig.key;
  return $(`<label class="switch"> <input type="checkbox" name="${name}"> <span class="slider round"></span> </label>`);
}

/**
 * Form Select View Generator
 *
 * @param {*} columnConfig
 * @returns
 */
const selectViewGenerator = (columnConfig) => {
  const name = columnConfig.key;
  const optionsHtml = columnConfig.options.map(x => `<option value="${x.value}">${x.name}</option>`).join('');
  return $(`<select name="${name}">${optionsHtml}</select>`);
}

/**
 * Form Button View Generator
 *
 * @param {*} buttonConfig
 * @returns
 */
const buttonViewGenerator = (buttonConfig) => {
  return $(`<button type="${buttonConfig.type}" class="action__${buttonConfig.key}">${buttonConfig.name}</button>`)
}


export {
  createForm,
  formViewGenerator,
  formButtonViewGenerator
}